import PagerItem from '@opentinyvue/vue-pager-item';
import Popover from '@opentinyvue/vue-popover';
import Loading from '@opentinyvue/vue-loading';
import { t } from '@opentinyvue/vue-locale';
import { defineComponent, $prefix, hooks, $props } from '@opentinyvue/vue-common';
import { IconChevronLeft, IconChevronRight, IconChevronDown } from '@opentinyvue/vue-icon';
import { emitEvent } from '@opentinyvue/utils';

function normalizeComponent(scriptExports, render, staticRenderFns, functionalTemplate, injectStyles, scopeId, moduleIdentifier, shadowMode) {
  var options = typeof scriptExports === "function" ? scriptExports.options : scriptExports;
  if (render) {
    options.render = render;
    options.staticRenderFns = staticRenderFns;
    options._compiled = true;
  }
  var hook;
  if (injectStyles) {
    hook = injectStyles;
  }
  if (hook) {
    if (options.functional) {
      options._injectStyles = hook;
      var originalRender = options.render;
      options.render = function renderWithStyleInjection(h, context) {
        hook.call(context);
        return originalRender(h, context);
      };
    } else {
      var existing = options.beforeCreate;
      options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
    }
  }
  return {
    exports: scriptExports,
    options
  };
}

function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t2 = arguments[e];
      for (var r in t2) ({}).hasOwnProperty.call(t2, r) && (n[r] = t2[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
var __vue2_script = defineComponent({
  name: $prefix + "Pager",
  components: {
    Pager: PagerItem,
    Popover,
    TinyIconChevronLeft: IconChevronLeft(),
    TinyIconChevronRight: IconChevronRight(),
    TinyIconChevronDown: IconChevronDown()
  },
  emits: ["update:currentPage", "update:pageSize", "size-change", "page-change", "before-change", "before-page-change", "current-change", "prev-click", "next-click"],
  props: _extends({}, $props, {
    accurateJumper: {
      type: Boolean,
      default: function _default() {
        return true;
      }
    },
    appendToBody: {
      type: Boolean,
      default: function _default2() {
        return true;
      }
    },
    currentPage: {
      type: Number,
      default: function _default3() {
        return 1;
      }
    },
    disabled: {
      type: Boolean,
      default: function _default4() {
        return false;
      }
    },
    hideOnSinglePage: Boolean,
    isBeforePageChange: Boolean,
    layout: String,
    mode: String,
    nextText: String,
    pageCount: Number,
    pageSize: {
      type: Number,
      default: function _default5() {
        return 10;
      }
    },
    pageSizes: {
      type: Array,
      default: function _default6() {
        return [10, 20, 30, 40, 50, 100];
      }
    },
    pagerCount: {
      type: Number,
      validator: function validator(value) {
        return (value | 0) === value && value > 2 && value < 22 && value % 2 === 1;
      },
      default: function _default7() {
        return 7;
      }
    },
    popperAppendToBody: {
      type: Boolean,
      default: function _default8() {
        return true;
      }
    },
    showTotalLoading: {
      type: Boolean,
      default: function _default9() {
        return false;
      }
    },
    customTotal: {
      type: [Boolean, String],
      default: function _default10() {
        return false;
      }
    },
    popperClass: String,
    prevText: String,
    total: Number,
    changeCompat: Boolean
  }),
  setup: function setup(props, _ref) {
    var emit = _ref.emit;
    var ref = hooks.ref, reactive = hooks.reactive, computed = hooks.computed, watch = hooks.watch, onMounted = hooks.onMounted, nextTick = hooks.nextTick, toRefs = hooks.toRefs;
    var sizesList = ref(null);
    var state = reactive({
      internalCurrentPage: 1,
      // 内部当前页码
      internalPageSize: 0,
      // 内部每页条数
      lastEmittedPage: -1,
      // 最后触发的页码
      userChangePageSize: false,
      // 用户是否改变了每页条数
      internalTotal: props.total,
      // 内部总条数
      showSizes: false,
      // 是否显示每页条数选择器
      jumperValue: 1
      // 跳转器输入值
    });
    var handleJumperFocus = function handleJumperFocus2(e) {
      var target = e.target;
      state.jumperValue = Number(target.value);
    };
    var handleJumperInput = function handleJumperInput2(e) {
      var target = e.target;
      var value = String(target.value);
      if (!value) {
        state.jumperValue = 1;
      } else if (/^\d+$/.test(value)) {
        state.jumperValue = Number(value) || 1;
      }
      target.value = String(state.jumperValue);
    };
    var handleJumperChange = function handleJumperChange2() {
      parseJumperValue();
      var callback = function callback2() {
        handleJumperClick();
      };
      var rollback = function rollback2() {
        state.jumperValue = state.internalCurrentPage;
      };
      var newPage = state.jumperValue;
      var currentPage = state.internalCurrentPage;
      if (props.isBeforePageChange && newPage !== currentPage) {
        var params = {
          newPage,
          currentPage,
          callback,
          rollback
        };
        beforePagerChangeHandler(params);
      } else {
        callback();
      }
    };
    var handleJumperClick = function handleJumperClick2() {
      if (!canJumperGo()) return;
      state.internalCurrentPage = getValidCurrentPage(Number(state.jumperValue));
      emitChange();
    };
    var parseJumperValue = function parseJumperValue2() {
      var value = Number(String(state.jumperValue).split(/[^0-9-+.]/).join(""));
      if (isNaN(value)) {
        value = 1;
      }
      value = Number(value.toFixed(0));
      if (internalPageCount.value) {
        if (value >= internalPageCount.value) {
          state.jumperValue = internalPageCount.value;
        } else if (value <= 1) {
          state.jumperValue = 1;
        } else {
          state.jumperValue = value;
        }
      }
    };
    var handleSizeChange = function handleSizeChange2(val) {
      if (val !== state.internalPageSize) {
        var callback = function callback2() {
          if (!beforeChangeHandler()) {
            return false;
          }
          state.internalPageSize = val;
          state.userChangePageSize = true;
          state.showSizes = false;
          emit("update:pageSize", val);
          emit("size-change", val);
          emit("page-change", {
            currentPage: state.internalCurrentPage,
            pageSize: val,
            total: state.internalTotal
          });
          if (Array.isArray(sizesList.value)) {
            sizesList.value[0].state.showPopper = false;
          }
        };
        if (props.isBeforePageChange) {
          var newPageSize = val;
          var currentPageSize = state.internalPageSize;
          var params = {
            newPageSize,
            currentPageSize,
            callback
          };
          beforeSizeChangeHandler(params);
        } else {
          callback();
        }
      }
    };
    var canJumperGo = function canJumperGo2() {
      var inputValue = Number(state.jumperValue || 0);
      var currentPage = Number(state.internalCurrentPage || 0);
      return props.accurateJumper ? inputValue !== currentPage : true;
    };
    var beforeSizeChangeHandler = function beforeSizeChangeHandler2(params) {
      var newPageSize = params.newPageSize, currentPageSize = params.currentPageSize, callback = params.callback;
      var newPage = 1;
      var currentPage = state.internalCurrentPage;
      var temp = {
        newPage,
        newPageSize,
        currentPage,
        currentPageSize,
        callback
      };
      emit("before-page-change", temp);
    };
    var beforePagerChangeHandler = function beforePagerChangeHandler2(params) {
      var newPage = params.newPage, currentPage = params.currentPage, callback = params.callback, rollback = params.rollback;
      var newPageSize = state.internalPageSize;
      var currentPageSize = state.internalPageSize;
      var temp = {
        newPage,
        newPageSize,
        currentPage,
        currentPageSize,
        callback,
        rollback
      };
      emit("before-page-change", temp);
    };
    var beforeJumperChangeHandler = function beforeJumperChangeHandler2(params) {
      var newPage = params.newPage, currentPage = params.currentPage, callback = params.callback, rollback = params.rollback;
      var newPageSize = state.internalPageSize;
      var currentPageSize = state.internalPageSize;
      var temp = {
        newPage,
        newPageSize,
        currentPage,
        currentPageSize,
        callback,
        rollback
      };
      emit("before-page-change", temp);
    };
    var copyEmit = function copyEmit2() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      emit.apply(void 0, [args[0]].concat(args.slice(1)));
    };
    var beforeChangeHandler = function beforeChangeHandler2(val) {
      if (val === void 0) {
        val = -1;
      }
      return emitEvent(copyEmit, "before-change", state.internalCurrentPage, null, val);
    };
    var handleCurrentChange = function handleCurrentChange2(val) {
      if (!beforeChangeHandler(val)) {
        return false;
      }
      state.internalCurrentPage = getValidCurrentPage(Number(val));
      state.userChangePageSize = true;
      emitChange();
    };
    var prev = function prev2() {
      var callback = function callback2() {
        if (props.disabled || !beforeChangeHandler(state.internalCurrentPage - 1)) {
          return false;
        }
        var newVal = state.internalCurrentPage - 1;
        state.internalCurrentPage = getValidCurrentPage(newVal);
        emit("prev-click", state.internalCurrentPage);
        emitChange();
      };
      if (props.isBeforePageChange) {
        var newPage = state.internalCurrentPage - 1;
        var temp = buildBeforePageChangeParam({
          newPage,
          callback
        });
        emit("before-page-change", temp);
      } else {
        callback();
      }
    };
    var next = function next2() {
      var callback = function callback2() {
        if (props.disabled || !beforeChangeHandler(state.internalCurrentPage + 1)) {
          return false;
        }
        var newVal = state.internalCurrentPage + 1;
        state.internalCurrentPage = getValidCurrentPage(newVal);
        emit("next-click", state.internalCurrentPage);
        emitChange();
      };
      if (props.isBeforePageChange) {
        var newPage = state.internalCurrentPage + 1;
        var temp = buildBeforePageChangeParam({
          newPage,
          callback
        });
        emit("before-page-change", temp);
      } else {
        callback();
      }
    };
    var buildBeforePageChangeParam = function buildBeforePageChangeParam2(param) {
      var currentPage = state.internalCurrentPage;
      var newPageSize = state.internalPageSize;
      var currentPageSize = state.internalPageSize;
      return Object.assign({
        currentPage,
        newPageSize,
        currentPageSize
      }, param);
    };
    var getValidCurrentPage = function getValidCurrentPage2(val) {
      val = parseInt(String(val), 10);
      var hasPageCount = typeof internalPageCount.value === "number";
      var resetVal;
      if (!hasPageCount) {
        if (isNaN(val) || val < 1) {
          resetVal = 1;
        }
      } else {
        if (val < 1) {
          resetVal = 1;
        } else if (internalPageCount.value && val > internalPageCount.value) {
          resetVal = internalPageCount.value;
        }
      }
      if (resetVal === void 0 && isNaN(val)) {
        resetVal = 1;
      } else if (resetVal === 0) {
        resetVal = 1;
      }
      return resetVal === void 0 ? val : resetVal;
    };
    var emitChange = function emitChange2() {
      nextTick(function() {
        if (state.internalCurrentPage !== state.lastEmittedPage || state.userChangePageSize) {
          emit("current-change", state.internalCurrentPage);
          emit("update:currentPage", state.internalCurrentPage);
          emit("page-change", {
            currentPage: state.internalCurrentPage,
            pageSize: state.internalPageSize,
            total: state.internalTotal
          });
          state.lastEmittedPage = state.internalCurrentPage;
          state.userChangePageSize = false;
        }
      });
    };
    var setTotal = function setTotal2(val) {
      state.internalTotal = val;
    };
    var totalText = computed(function() {
      if (typeof props.customTotal === "string") return props.customTotal;
      var totals = parseInt(String(props.total || 0));
      if (isNaN(totals)) return 0;
      var HUNDRED_THOUSAND = 1e5;
      var MILLION = 1e6;
      var TEN_MILLION = 1e7;
      if (totals < HUNDRED_THOUSAND) {
        return totals;
      } else if (totals < MILLION) {
        return t("ui.page.hundredThousand");
      } else if (totals < TEN_MILLION) {
        return t("ui.page.million");
      } else {
        return t("ui.page.tenMillion");
      }
    });
    var internalPageCount = computed(function() {
      if (typeof state.internalTotal === "number") {
        return Math.max(1, Math.ceil(state.internalTotal / state.internalPageSize));
      } else if (typeof props.pageCount === "number") {
        return Math.max(1, props.pageCount);
      }
      return null;
    });
    var internalLayout = computed(function() {
      var layout = "";
      if (props.mode && !props.layout) {
        props.mode === "number" && (layout = "sizes, prev, pager, next, jumper, total");
        props.mode === "simple" && (layout = "sizes, prev, next");
        props.mode === "complete" && (layout = "sizes, prev, pager, next, jumper,total");
        props.mode === "fixed" && (layout = "prev, pager, next");
      } else if (!props.mode && props.layout || props.mode && props.layout) {
        layout = props.layout;
      } else {
        layout = "prev, pager, next, jumper, total";
      }
      return layout;
    });
    watch(function() {
      return props.currentPage;
    }, function(curPage) {
      state.internalCurrentPage = getValidCurrentPage(curPage);
    }, {
      immediate: true
    });
    watch(internalPageCount, function(pageCount) {
      var oldCurPage = state.internalCurrentPage;
      if (pageCount && pageCount > 0 && oldCurPage === 0) {
        state.internalCurrentPage = 1;
      } else if (pageCount && oldCurPage > pageCount) {
        state.internalCurrentPage = pageCount === 0 ? 1 : pageCount;
        state.userChangePageSize && emitChange();
      }
      state.userChangePageSize = false;
    });
    watch(function() {
      return state.internalCurrentPage;
    }, function(curPage) {
      emit("update:currentPage", curPage);
      state.lastEmittedPage = -1;
    }, {
      immediate: true
    });
    watch(function() {
      return props.pageSize;
    }, function(pageSize) {
      state.internalPageSize = isNaN(pageSize) ? 10 : pageSize;
    }, {
      immediate: true
    });
    watch(function() {
      return props.total;
    }, function(total) {
      state.internalTotal = total;
    });
    onMounted(function() {
      if (document.querySelector('[data-tag="tiny-pager-total-loading"]')) {
        var target = document.querySelector('[data-tag="tiny-pager-total-loading"]');
        if (target) {
          Loading.service({
            target
          });
        }
      }
    });
    return _extends({}, toRefs(state), {
      // 方法
      t,
      handleJumperFocus,
      handleJumperInput,
      handleJumperChange,
      handleJumperClick,
      parseJumperValue,
      handleSizeChange,
      canJumperGo,
      beforeSizeChangeHandler,
      beforePagerChangeHandler,
      beforeJumperChangeHandler,
      copyEmit,
      beforeChangeHandler,
      handleCurrentChange,
      prev,
      next,
      buildBeforePageChangeParam,
      getValidCurrentPage,
      emitChange,
      setTotal,
      // 计算属性
      totalText,
      internalPageCount,
      internalLayout,
      // refs
      sizesList
    });
  }
});
var render = function render2() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return !(_vm.hideOnSinglePage && (!_vm.internalPageCount || _vm.internalPageCount <= 1)) ? _c("div", {
    staticClass: "text-right py-3 px-0 text-color-text-primary"
  }, [_vm._l(_vm.internalLayout.split(","), function(item, index) {
    return [item.trim() === "total" && typeof _vm.internalTotal === "number" ? _c("div", {
      key: "total" + index,
      staticClass: "inline-block align-middle text-xs h-7 leading-7 float-left"
    }, [_vm.showTotalLoading ? _c("div", {
      staticClass: "h-7 leading-7 text-xs text-color-text-primary"
    }, [_c("div", {
      staticClass: "inline-block align-baseline h-3.5 w-3.5 mr-1.5 top-0.5 [&_[data-tag=tiny-loading-icon]]:h-3.5 [&_[data-tag=tiny-loading-icon]]:w-3.5",
      attrs: {
        "data-tag": "tiny-pager-total-loading"
      }
    }), _c("span", {
      staticClass: "text-color-text-secondary"
    }, [_vm._v(_vm._s(_vm.t("ui.page.loadingTotals")))])]) : _c("div", {
      staticClass: "h-7 leading-7 text-xs text-color-text-primary"
    }, [_c("span", [_vm._v(_vm._s(_vm.t("ui.page.total")))]), _c("span", {
      staticClass: "my-0 mx-1"
    }, [_vm._v(" " + _vm._s(_vm.customTotal ? _vm.totalText : _vm.internalTotal) + " ")]), _c("span", [_vm._v(_vm._s(_vm.t("ui.page.item")))])])]) : item.trim() === "sizes" ? _c("div", {
      key: "sizes" + index,
      staticClass: "hidden sm:inline-block align-middle text-xs h-7 text-xs text-color-text-primary relative -top-px",
      attrs: {
        "data-tag": "tiny-pager-popover"
      }
    }, [_c("popover", {
      ref: "sizesList",
      refInFor: true,
      attrs: {
        "placement": "bottom-start",
        "append-to-body": _vm.popperAppendToBody,
        "trigger": "click",
        "popper-class": "w-24 sm:p-0 sm:!mt-1 sm:!mb-1 data-tag-pager-selector" + (_vm.popperClass ? " " + _vm.popperClass : ""),
        "visible-arrow": false,
        "disabled": _vm.disabled
      },
      on: {
        "show": function show($event) {
          _vm.showSizes = true;
        },
        "hide": function hide($event) {
          _vm.showSizes = false;
        }
      },
      scopedSlots: _vm._u([{
        key: "reference",
        fn: function fn() {
          return [_c("div", {
            staticClass: "m-0 ml-2",
            on: {
              "click": function click($event) {
                $event.stopPropagation();
              }
            }
          }, [_c("div", {
            ref: "pageSize",
            refInFor: true,
            class: ["min-w-[theme(spacing.18)] max-w-[theme(spacing.40)] relative text-left h-7 leading-7 border border-solid border-color-border rounded text-xs py-0 pr-1 pl-3 block whitespace-nowrap transition-[border] duration-300 outline-0 box-border select-none", _vm.showSizes ? "border-color-border-focus bg-color-fill-6 text-color-border-focus [&_svg]:rotate-180 [&_svg]:fill-color-brand-hover" : "", _vm.disabled ? "bg-color-border-disabled text-color-border cursor-not-allowed [&_svg]:fill-color-icon-disabled [&_svg]:cursor-not-allowed" : "bg-color-bg-1 text-color-text-primary hover:bg-color-border-disabled hover:border-color-border active:border-color-border-focus active:bg-color-fill-6 active:text-color-brand"]
          }, [_c("span", {
            staticClass: "text-xs mr-1 relative -top-px"
          }, [_vm._v(_vm._s(_vm.internalPageSize))]), _c("span", {
            staticClass: "relative -top-px"
          }, [_vm._v(_vm._s(_vm.t("ui.page.page")))]), _c("div", {
            staticClass: "w-7 h-7 leading-7 relative float-right -top-px outline-0 box-border text-center overflow-hidden cursor-pointer"
          }, [_c("tiny-icon-chevron-down", {
            staticClass: "fill-color-text-primary text-sm absolute top-0 left-0 right-0 bottom-0 m-auto hover:fill-color-icon-hover transition-transform duration-300"
          })], 1)])])];
        },
        proxy: true
      }], null, true)
    }, [_c("div", {
      staticClass: "max-h-[theme(spacing.72)] overflow-y-auto overflow-x-hidden"
    }, [_c("ul", _vm._l(_vm.pageSizes, function(sizeItem) {
      return _c("li", {
        key: String(sizeItem),
        class: ["min-h-[theme(spacing.8)] py-0 px-2 leading-8 max-w-full cursor-pointer overflow-hidden text-ellipsis text-center whitespace-nowrap m-1 rounded", sizeItem === _vm.internalPageSize ? "text-color-brand bg-color-fill-6" : "hover:bg-color-bg-2 text-color-text-primary"],
        attrs: {
          "data-value": sizeItem,
          "title": String(sizeItem)
        },
        on: {
          "click": function click($event) {
            _vm.handleSizeChange(Number(sizeItem));
          }
        }
      }, [_vm._v(" " + _vm._s(sizeItem) + " ")]);
    }), 0)])])], 1) : item.trim() === "prev" ? _c("button", {
      key: "prev" + index,
      staticClass: "group min-w-[theme(spacing.7)] h-7 text-xs py-0 px-1 text-color-text-primary bg-color-bg-1 rounded-sm outline-0 ml-0 sm:ml-2 align-bottom cursor-pointer hover:border-color-icon-primary disabled:cursor-default",
      attrs: {
        "type": "button",
        "disabled": _vm.disabled || _vm.internalCurrentPage <= 1
      },
      on: {
        "click": _vm.prev
      }
    }, [_vm.prevText ? _c("span", {
      staticClass: "group-disabled:text-color-text-disabled group-disabled:cursor-not-allowed group-hover:text-color-icon-hover"
    }, [_vm._v(" " + _vm._s(_vm.prevText) + " ")]) : _c("tiny-icon-chevron-left", {
      staticClass: "align-sub group-disabled:fill-color-icon-disabled group-disabled:cursor-not-allowed group-hover:fill-color-icon-active"
    })], 1) : item.trim() === "pager" ? _c("pager", {
      key: "pager" + index,
      attrs: {
        "is-before-page-change": _vm.isBeforePageChange,
        "current-page": _vm.internalCurrentPage,
        "page-count": _vm.internalPageCount || 0,
        "pager-count": _vm.pagerCount,
        "disabled": _vm.disabled
      },
      on: {
        "before-page-change": _vm.beforePagerChangeHandler,
        "change": _vm.handleCurrentChange
      }
    }) : item.trim() === "next" ? _c("button", {
      key: "next" + index,
      staticClass: "group min-w-[theme(spacing.7)] h-7 text-xs py-0 px-1 text-color-text-primary bg-color-bg-1 rounded-sm outline-0 ml-0 sm:ml-2 align-bottom cursor-pointer hover:border-color-icon-primary disabled:cursor-default",
      attrs: {
        "type": "button",
        "disabled": _vm.disabled || _vm.internalCurrentPage === _vm.internalPageCount || _vm.internalPageCount === 0
      },
      on: {
        "click": _vm.next
      }
    }, [_vm.nextText ? _c("span", {
      staticClass: "group-disabled:text-color-text-disabled group-disabled:cursor-not-allowed group-hover:text-color-icon-hover"
    }, [_vm._v(" " + _vm._s(_vm.nextText) + " ")]) : _c("tiny-icon-chevron-right", {
      staticClass: "align-sub group-disabled:fill-color-icon-disabled group-disabled:cursor-not-allowed group-hover:fill-color-icon-active"
    })], 1) : item.trim() === "jumper" ? _c("div", {
      key: "jumper" + index,
      staticClass: "h-7 leading-7 inline-block align-middle text-xs"
    }, [_c("div", {
      staticClass: "text-[0] h-7"
    }, [_c("span", {
      staticClass: "text-xs pl-4 pr-2 text-color-text-primary"
    }, [_vm._v(_vm._s(_vm.t("ui.page.jump")))]), _c("input", {
      ref: "jumperInput",
      refInFor: true,
      staticClass: "w-8 h-7 text-center align-top rounded-sm inline-block border border-solid border-color-border hover:text-color-icon-primary hover:border-color-icon-primary text-color-text-primary text-xs transition-[border] duration-300 outline-0 box-border mr-0 focus:border-color-border-focus",
      attrs: {
        "type": "tel",
        "disabled": _vm.disabled
      },
      domProps: {
        "value": _vm.jumperValue
      },
      on: {
        "focus": _vm.handleJumperFocus,
        "input": _vm.handleJumperInput,
        "change": _vm.handleJumperChange
      }
    })])]) : item.trim() === "slot" ? _vm._t("default") : _vm._e()];
  })], 2) : _vm._e();
};
var staticRenderFns = [];
var __cssModules = {};
var __component__ = /* @__PURE__ */ normalizeComponent(__vue2_script, render, staticRenderFns, false, __vue2_injectStyles);
function __vue2_injectStyles(context) {
  for (var o in __cssModules) {
    this[o] = __cssModules[o];
  }
}
var mobileFirst = /* @__PURE__ */ function() {
  return __component__.exports;
}();

export { mobileFirst as default };
